import binascii
import hashlib
import os


def generate_trace_id(*args) -> int:
    # Create input string similar to Go implementation
    input_str = "".join([str(arg) for arg in args])

    # Generate SHA-256 hash
    hash_obj = hashlib.sha256(input_str.encode())
    hash_hex = hash_obj.hexdigest()

    # Take first 32 characters and convert to bytes
    trace_id_hex = hash_hex[:32]
    trace_id_bytes = binascii.unhexlify(trace_id_hex)

    # Create OpenTelemetry trace ID
    return int.from_bytes(trace_id_bytes, byteorder="big")


def generate_span_id(*args) -> int:
    # Create input string similar to Go implementation
    input_str = "".join([str(arg) for arg in args])

    # Generate SHA-256 hash
    hash_obj = hashlib.sha256(input_str.encode())
    hash_hex = hash_obj.hexdigest()

    # Take characters 16-32 of the hex hash (8 bytes)
    span_id_hex = hash_hex[16:32]
    span_id_bytes = binascii.unhexlify(span_id_hex)

    return int.from_bytes(span_id_bytes, byteorder="big")


def get_job_trace_id() -> int:
    """returns a unique trace id for the job"""
    return generate_trace_id(
        os.environ.get("GITHUB_REPOSITORY", ""),
        os.environ.get("GITHUB_WORKFLOW", ""),
        os.environ.get("GITHUB_JOB", ""),
        os.environ.get("GITHUB_RUN_ID", ""),
        os.environ.get("GITHUB_RUN_NUMBER", ""),
        os.environ.get("GITHUB_RUN_ATTEMPT", ""),
    )


def get_job_span_id() -> int:
    """returns a unique trace id for the job"""
    return generate_span_id(
        os.environ.get("GITHUB_REPOSITORY", ""),
        os.environ.get("GITHUB_WORKFLOW", ""),
        os.environ.get("GITHUB_JOB", ""),
        os.environ.get("GITHUB_RUN_ID", ""),
        os.environ.get("GITHUB_RUN_NUMBER", ""),
        os.environ.get("GITHUB_RUN_ATTEMPT", ""),
        os.environ.get("STEP_ID", ""),
    )


def get_resource_attributes() -> dict[str, str | int]:
    """Returns resource attributes appropriate for
    https://github.com/open-telemetry/opentelemetry-collector-contrib/tree/main/receiver/githubreceiver

    // Not merged into main yet
    https://github.com/krzko/opentelemetry-collector-contrib/blob/feat-add-githubactionseventreceiver/receiver/githubactionsreceiver/trace_event_handling.go
    """
    # We are running within github action
    if os.environ.get("GITHUB_ACTION"):
        return {
            "ci.github.owner": os.environ.get("GITHUB_REPOSITORY_OWNER", ""),
            "ci.github.repository": os.environ.get("GITHUB_REPOSITORY", ""),
            "ci.github.workflow": os.environ.get("GITHUB_WORKFLOW", ""),
            "ci.github.actor": os.environ.get("GITHUB_ACTOR", ""),
            "ci.github.base_ref": os.environ.get("GITHUB_BASE_REF", ""),
            "ci.github.ref": os.environ.get("GITHUB_REF", ""),
            "ci.github.sha": os.environ.get("GITHUB_SHA", ""),
            "ci.github.job_id": os.environ.get("GITHUB_JOB", ""),
            "ci.github.run_id": os.environ.get("GITHUB_RUN_ID", ""),
            "ci.github.run_number": os.environ.get("GITHUB_RUN_NUMBER", ""),
            "ci.github.run_attempt": os.environ.get("GITHUB_RUN_ATTEMPT", ""),
            "ci.github.event_name": os.environ.get("GITHUB_EVENT_NAME", ""),
            "ci.github.url": f"{os.environ.get('GITHUB_SERVER_URL')}/{os.environ.get('GITHUB_REPOSITORY')}/actions/runs/{os.environ.get('GITHUB_RUN_ID')}",
            "ci.github.step_id": os.environ.get("STEP_ID", ""),
        }
    return {}



import os
from typing import Literal


def parse_key_val(key_val_str: str) -> dict[str, str]:
    return dict(h.split("=") for h in key_val_str.split(",") if "=" in h)


OTEL_TRACES_EXPORTER: Literal["otlp", "console", "file"] | None = os.environ.get(  # type: ignore
    "OTEL_TRACES_EXPORTER"
)
OTEL_EXPORTER_OTLP_PROTOCOL: Literal["grpc", "http/protobuf"] | None = os.environ.get(  # type: ignore
    "OTEL_EXPORTER_OTLP_PROTOCOL"
)
_headers = os.getenv("OTEL_EXPORTER_OTLP_HEADERS")
OTEL_EXPORTER_OTLP_HEADERS: dict | None = parse_key_val(_headers) if _headers else None

OTEL_EXPORTER_OTLP_INSECURE: bool = (
    os.getenv("OTEL_EXPORTER_OTLP_INSECURE", "false").lower() == "true"
)
OTEL_EXPORTER_FILE_DIRECTORY: str | None = os.getenv("OTEL_EXPORTER_FILE_DIRECTORY")
OTEL_EXPORTER_OTLP_ENDPOINT: str | None = os.getenv("OTEL_EXPORTER_OTLP_ENDPOINT")
OTEL_SERVICE_NAME: str | None = os.getenv(
    "OTEL_SERVICE_NAME", "github-action" if os.getenv("GITHUB_ACTION") else "otel-ci"
)
OTEL_SPAN_NAME: str | None = os.getenv("OTEL_SPAN_NAME")
_otel_resource_attributes = os.getenv("OTEL_RESOURCE_ATTRIBUTES")
OTEL_RESOURCE_ATTRIBUTES: dict[str, str] | None = (
    parse_key_val(_otel_resource_attributes) if _otel_resource_attributes else None
)



import argparse
import pathlib

from otel_ci import settings
from otel_ci.exec import exec_command
from otel_ci.logging import configure_logging
from otel_ci.replay import replay
from otel_ci.trace import configure_tracer

parser = argparse.ArgumentParser(description="A CLI tool")
parser.add_argument(
    "--service",
    "-s",
    type=str,
    dest="service_name",
    help="The name of the service [default:otel-cli]",
)
parser.add_argument(
    "--name",
    "-n",
    type=str,
    dest="span_name",
    help="The name of the span [default: command]",
)
parser.add_argument(
    "--resource-attributes",
    "-r",
    type=str,
    help="Resource attributes in key=value format, separated by commas",
)
parser.add_argument(
    "--protocol", type=str, help="Desired OTLP protocol: grpc,http/protobuf"
)
parser.add_argument(
    "--traces-exporter",
    "-e",
    type=str,
    help="The traces exporter to use: otlp, console, file",
)
parser.add_argument(
    "--output-directory",
    "-o",
    type=str,
    help="Export traces to directory (only for file exporter)",
)
parser.add_argument(
    "-v",
    "--verbose",
    action="count",
    default=0,
    dest="verbosity_level",
    help="Increase output verbosity (can be used multiple times)",
)

subparsers = parser.add_subparsers(dest="subcommand")
# Replay command
subparsers.add_parser("replay", help="Replay exported traces to an OTLP endpoint")

# Exec command
exec_parser = subparsers.add_parser("exec", help="Runs a command within a span")
exec_parser.add_argument("command", help="The command to run")


def main():
    args, other = parser.parse_known_args()

    configure_logging(args.verbosity_level)

    if args.service_name:
        settings.OTEL_SERVICE_NAME = args.service_name
    if args.resource_attributes:
        settings.OTEL_RESOURCE_ATTRIBUTES = settings.parse_key_val(
            args.resource_attributes
        )
    if args.protocol:
        settings.OTEL_EXPORTER_OTLP_PROTOCOL = args.protocol
    if args.traces_exporter:
        settings.OTEL_TRACES_EXPORTER = args.traces_exporter
    if args.output_directory:
        settings.OTEL_EXPORTER_FILE_DIRECTORY = args.output_directory
    if args.subcommand:
        configure_tracer()
        match args.subcommand:
            case "exec":
                if not args.command:
                    raise argparse.ArgumentError(None, "exec: No command provided")
                command = [args.command] + other
                exec_command(command, span_name=args.span_name)
            case "replay":
                replay(pathlib.Path(settings.OTEL_EXPORTER_FILE_DIRECTORY))
            case _:
                raise ValueError(f"Unknown subcommand: {args.subcommand}")
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import logging

logger = logging.getLogger("otel-cli")


def configure_logging(verbosity_level: int) -> None:
    if verbosity_level == 0:
        logging.basicConfig(level=logging.WARNING)
    elif verbosity_level == 1:
        logging.basicConfig(level=logging.INFO)
    elif verbosity_level >= 2:
        logging.basicConfig(level=logging.DEBUG)



import datetime
import json
import pathlib

from opentelemetry import trace
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import ReadableSpan

from otel_ci.logging import logger
from otel_ci.trace import get_exporter


def replay(output_directory: pathlib.Path) -> None:
    """Replay the"""
    otlp_exporter = get_exporter()
    spans = []
    if not output_directory.exists():
        raise ValueError("Trace directory %s does not exist", output_directory)

    for f in output_directory.glob("*.trace"):
        span_data = json.loads(f.read_text())
        # Convert ISO time to datetime
        start_time = datetime.datetime.fromisoformat(
            span_data["start_time"].replace("Z", "+00:00")
        )
        end_time = datetime.datetime.fromisoformat(
            span_data["end_time"].replace("Z", "+00:00")
        )

        # Generate trace and span IDs if not present
        trace_id = int(span_data["context"]["trace_id"], 16)
        span_id = int(span_data["context"]["span_id"], 16)

        # Create a resource (optional, but recommended)
        resource = Resource.create(
            {**(span_data.get("resource", {}).get("attributes", {}))}
        )

        # Create a ReadableSpan
        status_code = span_data.get("status", {}).get("status_code", "UNSET")
        status = trace.Status(
            status_code=trace.StatusCode[status_code],
            description=span_data.get("status", {}).get("description"),
        )
        sdk_span = ReadableSpan(
            name=span_data["name"],
            context=trace.SpanContext(trace_id, span_id, is_remote=False),
            parent=None,
            resource=resource,
            attributes=span_data.get("attributes", {}),
            events=[],  # Convert events if needed
            links=[],
            start_time=int(start_time.timestamp() * 1e9),
            end_time=int(end_time.timestamp() * 1e9),
            status=status,
        )
        spans.append(sdk_span)

        logger.debug("original_sdk_span: %s", span_data)
        logger.debug("parsed_sdk_span: %s", sdk_span.to_json())

    otlp_exporter.export(spans)



import pathlib
import time
from typing import Any

from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
    OTLPSpanExporter as GRPCSpanExporter,
)
from opentelemetry.exporter.otlp.proto.http.trace_exporter import (
    OTLPSpanExporter as HTTPSpanExporter,
)
from opentelemetry.sdk.resources import SERVICE_NAME, Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import (
    ConsoleSpanExporter,
    SimpleSpanProcessor,
    SpanExporter,
)

from otel_ci import settings
from otel_ci.github import get_resource_attributes

otlp_exporter: SpanExporter | None = None


def get_otlp_exporter() -> SpanExporter:
    """
    Dynamically select OTLP exporter based on configuration.

    Support exporters:
    - 'otlp': Uses OTLP exporter
    - 'console': Uses console exporter
    - 'file': Uses file exporter
    - 'none': Uses no exporter

    Supported OTLP protocols:
    - 'grpc': Uses gRPC exporter
    - 'http/protobuf': Uses HTTP protobuf exporter
    - 'http/json': Uses HTTP JSON exporter (falls back to protobuf if not specified)
    """
    exporter = settings.OTEL_TRACES_EXPORTER
    protocol = settings.OTEL_EXPORTER_OTLP_PROTOCOL

    # Prepare exporter configuration
    exporter_kwargs: dict[str, Any] = {}

    if settings.OTEL_EXPORTER_OTLP_ENDPOINT:
        exporter_kwargs["endpoint"] = settings.OTEL_EXPORTER_OTLP_ENDPOINT
    if settings.OTEL_EXPORTER_OTLP_HEADERS:
        exporter_kwargs["headers"] = settings.OTEL_EXPORTER_OTLP_HEADERS
    if settings.OTEL_EXPORTER_OTLP_INSECURE:
        exporter_kwargs["insecure"] = settings.OTEL_EXPORTER_OTLP_INSECURE

    # Select exporter based on protocol
    if protocol == "grpc" and exporter == "otlp":
        return GRPCSpanExporter(**exporter_kwargs, timeout=1000, insecure=True)
    elif protocol in ["http/protobuf", "http/json"] and exporter == "otlp":
        return HTTPSpanExporter(**exporter_kwargs)
    elif exporter == "console":
        return ConsoleSpanExporter()
    elif exporter == "file":
        output_path = settings.OTEL_EXPORTER_FILE_DIRECTORY
        if not output_path:
            raise ValueError(
                "-o <output_directory> or `OTEL_EXPORTER_FILE_DIRECTORY` must be specified when using file export protocol"
            )
        pathlib.Path(output_path).mkdir(parents=True, exist_ok=True)
        filename = pathlib.Path(output_path) / f"{time.time()}.trace"
        output_file = open(filename, mode="w")
        return ConsoleSpanExporter(out=output_file)
    else:
        return SpanExporter()


def configure_tracer() -> None:
    global otlp_exporter
    assert settings.OTEL_SERVICE_NAME
    resource_attributes = {
        SERVICE_NAME: settings.OTEL_SERVICE_NAME,
        **get_resource_attributes(),
    }
    resource = Resource(attributes=resource_attributes)
    otlp_exporter = get_otlp_exporter()
    trace.set_tracer_provider(TracerProvider(resource=resource))
    span_processor = SimpleSpanProcessor(otlp_exporter)
    trace.get_tracer_provider().add_span_processor(span_processor)  # type: ignore


def get_exporter() -> SpanExporter:
    assert otlp_exporter
    return otlp_exporter



import os
import subprocess

from opentelemetry import trace
from opentelemetry.sdk.resources import PROCESS_COMMAND, PROCESS_COMMAND_ARGS
from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator

from otel_ci import github
from otel_ci.logging import logger


def exec_command(command: list[str], span_name: str | None = None) -> None:
    """Executes a command within a trace span"""
    logger.info("Executing command: ", command)
    tracer = trace.get_tracer(__name__)
    command_str = " ".join(command)

    # Create a TraceContextTextMapPropagator instance
    propagator = TraceContextTextMapPropagator()

    # Extract the trace context from the TRACEPARENT header
    trace_parent = os.environ.get("TRACEPARENT")
    carrier = {}
    if trace_parent:
        carrier["traceparent"] = trace_parent
    else:
        if os.environ.get("GITHUB_ACTION"):
            carrier["traceparent"] = (
                f"00-{github.get_job_trace_id()}-{github.get_job_span_id}-01"
            )
    context = propagator.extract(carrier)

    with tracer.start_as_current_span(
        span_name or " ".join(command),
        attributes={PROCESS_COMMAND: command[0], PROCESS_COMMAND_ARGS: command},
        context=context,
    ) as span:
        obj: dict[str, str] = {}
        propagator.inject(obj)
        env = os.environ.copy()
        env["TRACEPARENT"] = obj["traceparent"]

        try:
            code = subprocess.call(command_str, shell=True, env=env)
        except KeyboardInterrupt:
            code = 130
        if code != 0:
            span.set_status(
                status=trace.StatusCode.ERROR,
                description=f"Command exited with code {code}",
            )
        else:
            span.set_status(status=trace.StatusCode.OK)
